/**************************************************************
 *
 * prc2bin.c
 *
 * Extracts every resources from PalmOS PRC file
 * Written by E.Sundaw <sundaw@yahoo.com>, 1999
 *
 *************************************************************/
#pragma pack(1)

#include <stdio.h>
#include <stdlib.h>
#if defined(__TOS__)
#include <string.h>
#endif
#include "prc.h"

unsigned short rgetw (FILE *fp)
{
    return ((fgetc (fp) << 8) | fgetc (fp));
}

unsigned long rgetl (FILE *fp)
{
    return (((unsigned long)fgetc (fp) << 24) | ((unsigned long)fgetc (fp) << 16) | (fgetc (fp) << 8) | fgetc (fp));
}

unsigned long filelength (FILE *fp)
{
    unsigned long l, c = ftell (fp);
    fseek (fp, 0, SEEK_END);
    l = ftell (fp);
    fseek (fp, c, SEEK_SET);
    return l;
}

int main (int argc, char *argv [])
{
    PRC_HDR hdr;
    PRC_RESOURCE_HDR *res;
    FILE *fp, *os;
    long i, len;
    char filename [260], *buf;
    
    if (argc != 2) {
        printf ("Usage: %s <filename>\n\n", argv [0]);
	return 1;
    }
    
    if ((fp = fopen (argv [1], "rb")) == NULL) {
        perror ("fopen()");
	return 1;
    }
    
    fread (&hdr.name, 32, 1, fp);
    hdr.flags = rgetw (fp);
    hdr.version = rgetw (fp);
    hdr.create_time = rgetl (fp);
    hdr.mod_time = rgetl (fp);
    hdr.backup_time = rgetl (fp);
    hdr.mod_num = rgetl (fp);
    hdr.app_info = rgetl (fp);
    hdr.sort_info = rgetl (fp);
    hdr.type = rgetl (fp);
    hdr.id = rgetl (fp);
    hdr.unique_id_seed = rgetl (fp);
    hdr.next_record_list = rgetl (fp);
    hdr.num_records = rgetw (fp);
    res = (PRC_RESOURCE_HDR *) malloc (sizeof (PRC_RESOURCE_HDR) * hdr.num_records);
    for (i = 0; i < hdr.num_records; i++) {
        fread (res [i].name, 4, 1, fp);
	res [i].id = rgetw (fp);
	res [i].offset = rgetl (fp);
    }
    
    for (i = 0; i < hdr.num_records; i++) {
        len = (i < hdr.num_records - 1) ? res [i + 1].offset - res [i].offset : filelength (fp) - res [i].offset;
	sprintf (filename, "%c%c%c%c%04x.bin", res [i].name [0], res [i].name [1], res [i].name [2], res [i].name [3], res [i].id);
	printf ("Writing %s from offset 0x%08lx (name=%c%c%c%c id=%d)\n", filename, res [i].offset, res [i].name [0], res [i].name [1], res [i].name [2], res [i].name [3], res [i].id);
	if ((os = fopen (filename, "wb")) == NULL) {
	    perror ("fopen()");
	    return 1;
	}
	buf = malloc (len);
	fseek (fp, res [i].offset, SEEK_SET);
	fread (buf, len, 1, fp);
	fwrite (buf, len, 1, os);
	fclose (os);
	free (buf);
    }
#if defined(__TOS__)
    {
	char *p;
	strcpy (filename, argv [1]);
	p = strrchr (filename, '.');
	if (p != NULL)
	    *p = '\0';
	strcat (p, ".hdr");
    }
#else
    sprintf (filename, "%s.hdr", argv [1]);
#endif
    if ((os = fopen (filename, "wb")) == NULL) {
	perror ("fopen()");
	return 1;
    }
    buf = malloc (sizeof (hdr));
    fseek (fp, 0, SEEK_SET);
    fread (buf, sizeof (hdr), 1, fp);
    fwrite (buf, sizeof (hdr), 1, os);
    fclose (os);
    free (buf);

    printf ("%d resources and header written.\n", hdr.num_records);
    printf ("\n");
    fclose (fp);
    return 0;
}
